from src.pipeline.base import PipelineBase
from typing import Dict, Any
from queue import Queue
from src.utils import get_module_logger

logger = get_module_logger(__name__)

class ASRPipeline(PipelineBase):
    """
    管道类
    实现具体的处理逻辑
    """
    def __init__(self, *args, **kwargs):
        """
        初始化管道
        """
        super().__init__(*args, **kwargs)
        self._config: Dict[str, Any] = {}
        self._functor_dict: Dict[str, Any] = {}
        self._subqueue_dict: Dict[str, Any] = {}
        self._is_baked: bool = False

    def set_config(self, config: Dict[str, Any]) -> None:
        """
        设置配置
        参数:
            config: Dict[str, Any] 配置
        """
        self._config = config

    def set_audio_binary(self, audio_binary: AudioBinary) -> None:
        """
        设置音频二进制存储单元
        参数:
            audio_binary: 音频二进制
        """
        self._audio_binary = audio_binary

    def set_models(self, models: Dict[str, Any]) -> None:
        """
        设置模型
        """
        self._models = models

    def bake(self) -> None:
        """
        烘焙管道
        """
        self._init_functor()
        self._is_baked = True

    def _init_functor(self) -> None:
        """
        初始化函数
        """
        try:
            from src.functor import functorFactory
            # 加载VAD、asr、spk functor
            self._functor_dict["vad"] = functorFactory.make_functor(
                functor_name = "vad", 
                config = self._config, 
                models = self._models
            )
            self._functor_dict["asr"] = functorFactory.make_functor(
                functor_name = "asr", 
                config = self._config, 
                models = self._models
            )
            self._functor_dict["spk"] = functorFactory.make_functor(
                functor_name = "spk", 
                config = self._config, 
                models = self._models
            )

            # 初始化子队列
            self._subqueue_dict["vad2asr"] = Queue()
            self._subqueue_dict["vad2spk"] = Queue()
            self._subqueue_dict["asrend"] = Queue()
            self._subqueue_dict["spkend"] = Queue()

            # 设置子队列的输入队列
            self._functor_dict["vad"].set_input_queue(self._input_queue)
            self._functor_dict["asr"].set_input_queue(self._subqueue_dict["vad2asr"])
            self._functor_dict["spk"].set_input_queue(self._subqueue_dict["vad2spk"])

            # 设置回调函数——放置到对应队列中
            self._functor_dict["vad"].add_callback(self._subqueue_dict["vad2asr"].put)
            self._functor_dict["vad"].add_callback(self._subqueue_dict["vad2spk"].put)

            # 构造带回调函数的put
            def put_with_checkcallback(queue: Queue, callback: Callable) -> None:
                """
                带回调函数的put
                """
                def put_with_check(data: Any) -> None:
                    queue.put(data)
                    callback()
                return put_with_check

            self._functor_dict["asr"].add_callback(put_with_checkcallback(self._subqueue_dict["asrend"], self._check_result))
            self._functor_dict["spk"].add_callback(put_with_checkcallback(self._subqueue_dict["spkend"], self._check_result))

        except ImportError:
            raise ImportError("functorFactory引入失败,ASRPipeline无法完成初始化") 

    def get_config(self) -> Dict[str, Any]:
        """
        获取配置
        返回:
            Dict[str, Any] 配置
        """
        return self._config

    def process(self, data: Any) -> Any:
        """
        处理数据
        参数:
            data: 输入数据
        返回:
            处理结果
        """
        # 子类实现具体的处理逻辑
        self._input_queue.put(data)

    def run(self) -> None:
        """
        运行管道
        """
        if not self._is_baked:
            raise RuntimeError("管道未烘焙，无法运行")

        # 运行所有functor
        for functor_name, functor in self._functor_dict.items():
            logger.info(f"运行{functor_name}functor")
            self._functor_dict[functor_name].run()

        # 运行管道
        if not self._input_queue:
            raise RuntimeError("输入队列未设置")

        # 设置管道运行状态
        self._is_running = True
        self._stop_event = False
        self._thread = threading.current_thread()
        logger.info("ASR管道开始运行")

        while self._is_running and not self._stop_event:
            try:
                # 从队列获取数据
                try:
                    data = self._input_queue.get(timeout=self._queue_timeout)
                    # 检查是否是结束信号
                    if data is None:
                        logger.info("收到结束信号，管道准备停止")
                        self._stop()
                        self._input_queue.task_done()  # 标记结束信号已处理
                        break
                    
                    # 处理数据
                    self.process(data)

                    # 标记任务完成
                    self._input_queue.task_done()
                    
                except Empty:
                    # 队列获取超时，继续等待
                    continue
                    
            except Exception as e:
                logger.error(f"管道处理数据出错: {str(e)}")
                continue

        logger.info("管道停止运行")
        
    def _check_result(self, result: Any) -> None:
        """
        检查结果
        """
        # 若asr和spk队列中都有数据，则合并数据
        if self._subqueue_dict["asrend"].qsize() & self._subqueue_dict["spkend"].qsize():
            asr_data = self._subqueue_dict["asrend"].get()
            spk_data = self._subqueue_dict["spkend"].get()
            # 合并数据
            result = {
                "asr_data": asr_data,
                "spk_data": spk_data
            }
            # 通知回调函数
            self._notify_callbacks(result)

    def stop(self) -> None:
        """
        停止管道
        """
        self._is_running = False
        self._stop_event = True
        for functor_name, functor in self._functor_dict.items():
            logger.info(f"停止{functor_name}functor")
            functor.stop()
        logger.info("子Functor停止")
